import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn import metrics
from sklearn.datasets import make_classification
from sklearn.model_selection import StratifiedShuffleSplit
from sklearn.tree import DecisionTreeClassifier
from sklearn.metrics import accuracy_score, f1_score, precision_score, recall_score, classification_report, confusion_matrix, make_scorer
from datetime import datetime
import joblib
import sys
import os
from sklearn.model_selection import cross_val_score
from Bio import SeqIO
import pickle
import time
import os.path

# file with lineage assignments
lineage_file = sys.argv[1]
# file with sequences
sequence_file = sys.argv[2]
# how much of the data will be used for testing, instead of training
testing_percentage = 0.0000000001

relevant_positions = pickle.load(open(sys.argv[5], 'rb'))
relevant_positions.add(0)

# the path to the reference file. 
# This reference sequence must be the same as is used in the pangolearn script!!
referenceFile = sys.argv[3]

# data storage
dataList = []
# dict for lookup efficiency
indiciesToKeep = dict()

referenceId = "Wuhan/WH04/2020"
referenceSeq = ""

idToLineage = dict()
idToSeq = dict()

mustKeepIds = []
mustKeepLineages = []


# function for handling weird sequence characters
def clean(x, loc):
	x = x.upper()
	
	if x == 'T' or x == 'A' or x == 'G' or x == 'C' or x == '-':
		return x

	if x == 'U':
		return 'T'

	# otherwise return value from reference
	return referenceSeq[loc]

def findReferenceSeq():
	with open(referenceFile) as f:
		currentSeq = ""

		for line in f:
			if ">" not in line:
				currentSeq = currentSeq + line.strip()

	f.close()
	return currentSeq


def getDataLine(seqId, seq):
	dataLine = []
	dataLine.append(seqId)

	newSeq = ""

	# for each character in the sequence
	for index in range(len(seq)):
		newSeq = newSeq + clean(seq[index], index)

	dataLine.append(newSeq)
	
	return dataLine


def readInAndFormatData():

	# add the data line for the reference seq
	idToLineage[referenceId] = "A"
	dataList.append(getDataLine(referenceId, referenceSeq))

	# create a dictionary of sequence ids to their assigned lineages
	lineage_designations = pd.read_csv(lineage_file, delimiter=",", dtype=str)
	for index, row in lineage_designations.iterrows(): 
		idToLineage[row["sequence_name"]] = row["lineage"]

	seq_dict = {rec.id : rec.seq for rec in SeqIO.parse(sequence_file, "fasta")}

	print("files read in, now processing")

	for key in seq_dict.keys():
		if key in idToLineage:
			dataList.append(getDataLine(key, seq_dict[key]))
		else:
			print("unable to find the lineage classification for: " + key)


# find columns in the data list which always have the same value
def findColumnsWithoutSNPs():

	# for each index in the length of each sequence
	for index in range(len(dataList[0][1])):
		keep = False

		# loop through all lines
		for line in dataList:

			# if there is a difference somewhere, then we want to keep it
			if dataList[0][1][index] != line[1][index] or index == 0:
				keep = True
				break

		# otherwise, save it
		if keep and index in relevant_positions:
			indiciesToKeep[index] = True


# remove columns from the data list which don't have any SNPs. We do this because
# these columns won't be relevant for a logistic regression which is trying to use
# differences between sequences to assign lineages
def removeOtherIndices(indiciesToKeep):

	# instantiate the final list
	finalList = []

	indicies = list(indiciesToKeep.keys())
	indicies.sort()

	# while the dataList isn't empty
	while len(dataList) > 0:

		# pop the first line
		line = dataList.pop(0)
		seqId = line.pop(0)

		line = line[0]
		# initialize the finalLine
		finalLine = []

		for index in indicies:
			if index == 0:
				# if its the first index, then that's the lineage assignment, so keep it
				finalLine.append(seqId)
			else:
				# otherwise keep everything at the indices in indiciesToKeep
				finalLine.append(line[index])

		# save the finalLine to the finalList
		finalList.append(finalLine)

	# return
	return finalList

def allEqual(list):
		entries = dict()

		for i in list:
			if i not in entries:
				entries[i] = True

		return len(entries) == 1

def removeAmbiguous():
	idsToRemove = set()
	lineMap = dict()
	idMap = dict()

	for line in dataList:
		keyString = ",".join(line[1:])

		if keyString not in lineMap:
			lineMap[keyString] = []
			idMap[keyString] = []
 
		if line[0] in idToLineage:
			lineMap[keyString].append(idToLineage[line[0]])
			idMap[keyString].append(line[0])
		else:
			print("diagnostics")
			print(line[0])
			print(keyString)
			print(line)
	for key in lineMap:
		if not allEqual(lineMap[key]):

			skipRest = False

			# see if any protected lineages are contained in the set, if so keep those ids
			for lineage in lineMap[key]:
				if lineage in mustKeepLineages:
					skipRest = True

					for i in idMap[key]:
						if lineage != idToLineage[i] and i not in mustKeepIds:
							idsToRemove.add(i)

			# none of the lineages are protected, fire at will
			if not skipRest:

				lineageToCounts = dict()

				aLineage = False
				# find most common lineage
				for lineage in lineMap[key]:
					if lineage not in lineageToCounts:
						lineageToCounts[lineage] = 0

					lineageToCounts[lineage] = lineageToCounts[lineage] + 1
					aLineage = lineage

				m = aLineage
				for lineage in lineageToCounts:
					if lineageToCounts[lineage] > lineageToCounts[m]:
						m = lineage


				for i in idMap[key]:
					if m != idToLineage[i]:
						idsToRemove.add(i)

	newList = []

	print("keeping indicies:")

	for line in dataList:
		if line[0] not in idsToRemove:
			print(line[0])
			line[0] = idToLineage[line[0]]
			newList.append(line)

	return newList


print("reading in data " + datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), flush=True)

referenceSeq = findReferenceSeq()

readInAndFormatData()

print("processing snps, formatting data " + datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), flush=True)

findColumnsWithoutSNPs()

dataList = removeOtherIndices(indiciesToKeep)

print("# sequences before blacklisting")
print(len(dataList))

dataList = removeAmbiguous()

print("# sequences after blacklisting")
print(len(dataList))

# headers are the original genome locations
headers = list(indiciesToKeep.keys())
headers[0] = "lineage"

print("setting up training " + datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), flush=True)

pima = pd.DataFrame(dataList, columns=headers)

# nucleotide symbols which can appear
categories = ['A', 'C', 'G', 'T', '-']

# one hot encoding of all headers other than the first which is the lineage
dummyHeaders = headers[1:]

# add extra rows to ensure all of the categories are represented, as otherwise 
# not enough columns will be created when we call get_dummies
for i in categories:
	line = [i] * len(dataList[0])
	pima.loc[len(pima)] = line

# get one-hot encoding
pima = pd.get_dummies(pima, columns=dummyHeaders)

# get rid of the fake data we just added
pima.drop(pima.tail(len(categories)).index, inplace=True)

feature_cols = list(pima)
print(feature_cols)

# remove the last column from the data frame. This is because we are trying to predict these values.
h = feature_cols.pop(0)
X = pima[feature_cols]
y = pima[h]

X_train,X_test,y_train,y_test=train_test_split(X,y,test_size=testing_percentage,random_state=0)

print("training " + datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), flush=True)

header_out = os.path.join(sys.argv[4],"decisionTreeHeaders_v1.joblib")
joblib.dump(headers, header_out, compress=('lzma', 9))

# instantiate the random forest with 50 trees
dt = DecisionTreeClassifier()

# fit the model
dt.fit(X,y)

print("testing " + datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), flush=True)

# classify the test data
y_pred=dt.predict(X_test)

print(y_pred)

# get the scores from these predictions
y_scores = dt.predict_proba(X_test)

print("generating statistics " + datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), flush=True)

#print the confusion matrix
print("--------------------------------------------")
print("Confusion Matrix")
cnf_matrix = metrics.confusion_matrix(y_test, y_pred)
print(cnf_matrix)

print("--------------------------------------------")
print("Classification report")
print(metrics.classification_report(y_test, y_pred, digits=3))

# save the model files to compressed joblib files
# using joblib instead of pickle because these large files need to be compressed
model_out = os.path.join(sys.argv[4],"decisionTree_v1.joblib")
joblib.dump(dt,  model_out, compress=('lzma', 9))


print("model files created", flush=True)

# this method is used below when running 10-fold cross validation. It ensures
# that the per-lineage statistics are generated for each cross-fold
def classification_report_with_accuracy_score(y_true, y_pred):
	print("--------------------------------------------")
	print("Crossfold Classification Report")
	print(metrics.classification_report(y_true, y_pred, digits=3))
	return accuracy_score(y_true, y_pred)

# optionally, run 10-fold cross validation (comment this out if not needed as it takes a while to run)
# cross_validation_scores = cross_val_score(dt, X=X, y=y, cv=10, scoring=make_scorer(classification_report_with_accuracy_score))



from sklearn.tree import DecisionTreeClassifier
from sklearn.tree import export_text
import joblib
import sys
from sklearn.tree import _tree

sys.setrecursionlimit(4000)

loaded_model = joblib.load(sys.argv[1])
headers = joblib.load(sys.argv[2])
output_file = sys.argv[3]

finalHeaders = []

with open(output_file, 'r') as f:
	for line in f:
		if line[0] == "[" and len(line) > 1000:
			print(line)
			finalHeaders = line[1:-1].split(",")

			for i in range(len(finalHeaders)):
				finalHeaders[i] = finalHeaders[i].strip()
				finalHeaders[i] = finalHeaders[i].replace("'", "")

print(finalHeaders)

categories = ['A', 'C', 'G', 'T', 'N', '-']

classes = loaded_model.classes_


def getClass(weightArray):
	weightArray = weightArray[0]

	maxValue = 0
	maxIndex = 0

	for i in range(len(weightArray)):
		if weightArray[i] > maxValue:
			maxValue = weightArray[i]
			maxIndex = i

	return classes[maxIndex]


def reformatRule(sign, name, threshold):
	position = name.split("_")[0]
	letter = name.split("_")[1]

	symbol = "=="
	if sign == "<=":
		symbol = "!="

	return position + symbol + "'" + letter + "'"


outcomesToRules = dict()


def tree_to_code(tree, feature_names):
    tree_ = tree.tree_
    feature_name = [
        feature_names[i] if i != _tree.TREE_UNDEFINED else "undefined!"
        for i in tree_.feature
    ]

    prevRules = []

    def recurse(node, depth, prevRules):
        indent = "  " * depth
        if tree_.feature[node] != _tree.TREE_UNDEFINED:
            name = feature_name[node]
            threshold = tree_.threshold[node]

            prevRulesRight = prevRules.copy()
            prevRulesLeft = prevRules.copy()

            prevRulesLeft.append(reformatRule("<=", name, threshold))
            prevRulesRight.append(reformatRule(">", name, threshold))

            recurse(tree_.children_left[node], depth + 1, prevRulesLeft)
            recurse(tree_.children_right[node], depth + 1, prevRulesRight)
        else:
            rulesString = prevRules[0]
            for rule in prevRules[1:]:
            	rule = rule.strip()

            	rulesString = rulesString + "," + rule

            outcome = getClass(tree_.value[node])

            if outcome not in outcomesToRules:
            	outcomesToRules[outcome] = []

            print(outcome + "\t" + rulesString)

    recurse(0, 1, prevRules)

tree_to_code(loaded_model, finalHeaders)


def formatCommonRules(ruleSets):
	splitRuleSets = []
	totalRules = []
	commonRules = []

	for r in ruleSets:
		splitRuleSets.append(r.split(","))

		for i in r.split(","):
			totalRules.append(i)

	for i in totalRules:

		addRule = True
		for r in splitRuleSets:
			if i not in r:
				addRule = False

		if addRule:
			commonRules.append(i)

			for r in splitRuleSets:
				r.remove(i)

	retString = ""

	if len(commonRules) > 0:
		retString = commonRules[0]

		for c in commonRules[1:]:
			retString = retString + "," + c

		retString = retString + " AND "

	for s in splitRuleSets:
		if len(s) > 0:
			retString = retString + "(" + s[0]
			for r in s[1:]:
				retString = retString + "," + r

			retString = retString + ") OR "


	return retString[:-3]



#!/usr/bin/env python3

import csv
import datetime
import operator
# import argparse
from Bio import SeqIO


# def parse_args():
#     parser = argparse.ArgumentParser(description="""Pick a representative sample for each unique sequence""",
#                                     formatter_class=argparse.RawTextHelpFormatter)
#     parser.add_argument('--in-metadata', dest = 'in_metadata', required=True, help='CSV of containing sequence_name and nucleotide_variants columns, the latter being | separated list of variants')
#     parser.add_argument('--in-fasta', dest = 'in_fasta', required=True, help='FASTA of all input seqs')
#     parser.add_argument('--diff', dest = 'diff', required=True, type=int, help='Samples within distance DIFF of included others may be excluded by the downsampler')
#     parser.add_argument('--out-metadata', dest = 'out_metadata', required=True, help='CSV to write out')
#     parser.add_argument('--out-fasta', dest = 'out_fasta', required=True, help='FASTA to write downsampled seqs')
#     parser.add_argument('--outgroups', dest = 'outgroups', required=False, help='Lineage splits file containing representative outgroups to protect')
#     parser.add_argument('--downsample_date_excluded', action='store_true', help='Downsample from those excluded as outside date window')
#     parser.add_argument('--downsample_included', action='store_true', help='Downsample from all included sequences')
#     parser.add_argument('--downsample_lineage_size', type=int, default=None, help='Min size of lineages to downsample, if unspecified no lineage-aware downsampling')

#     args = parser.parse_args()
#     return args

def parse_outgroups(outgroup_file):
    """
    input is CSV, last column being the representative outgroups:
    """
    outgroups = []
    if not outgroup_file:
        return outgroups
    with open(outgroup_file, "r") as outgroup_handle:
        line = outgroup_handle.readline()
        while line:
            try:
                outgroup = line.strip().split(",")[-1]
                outgroups.append(outgroup)
            except:
                continue
            line = outgroup_handle.readline()
    return(outgroups)

def get_count_dict(in_metadata):
    count_dict = {}
    num_samples = 0
    with open(in_metadata,"r") as f:
        reader = csv.DictReader(f)
        for row in reader:
            num_samples += 1
            for var in row["nucleotide_variants"].split("|"):
                if var in count_dict:
                    count_dict[var] += 1
                else:
                    count_dict[var] = 1
    print("Found", len(count_dict), "variants")

    sorted_tuples = sorted(count_dict.items(), key=operator.itemgetter(1))
    count_dict = {k: v for k, v in sorted_tuples}
    return count_dict, num_samples

def get_lineage_dict(in_metadata, min_size):
    lineage_dict = {}
    if min_size is None:
        return lineage_dict

    with open(in_metadata,"r") as f:
        reader = csv.DictReader(f)
        for row in reader:
            if "lineage" in row:
                lin = row["lineage"]
                if lin in lineage_dict:
                    lineage_dict[lin].append(row["sequence_name"])
                else:
                    lineage_dict[lin] = [row["sequence_name"]]
    print("Found", len(lineage_dict), "lineages")

    small_lineages = [lin for lin in lineage_dict if len(lineage_dict[lin]) < min_size]
    for lin in small_lineages:
        del lineage_dict[lin]
    print("Found", len(lineage_dict), "lineages with at least", min_size, "representative sequences")

    return lineage_dict

def get_by_frequency(count_dict, num_samples, band=[0.1,1.0]):
    lower_bound = num_samples*band[0]
    upper_bound = num_samples*band[1]
    most_frequent = [k for k in count_dict if lower_bound < count_dict[k] <= upper_bound]
    print(len(most_frequent), "lie in frequency band", band)
    return most_frequent

def num_unique(muts1, muts2):
    u1 = [m for m in muts1 if m not in muts2]
    u2 = [m for m in muts2 if m not in muts1]
    return len(u1+u2)

def should_downsample_row(row, downsample_date_excluded=True, downsample_included=False, downsample_lineage_size=None, lineage_dict={}):
    if downsample_included and row["why_excluded"] in [None, "None", ""]:
        return True
    if downsample_date_excluded and row["why_excluded"] in [None, "None", ""] and "date_filter" in row and row["date_filter"].startswith("sample_date older than"):
        return True
    if downsample_lineage_size and row["lineage"] in lineage_dict:
        return True
    return False

def downsample(in_metadata, out_metadata, in_fasta, out_fasta, max_diff, outgroup_file, downsample_date_excluded, downsample_included, downsample_lineage_size):
    original_num_seqs = 0
    sample_dict = {}
    var_dict = {}

    count_dict, num_samples = get_count_dict(in_metadata)
    most_frequent = get_by_frequency(count_dict, num_samples, band=[0.05,1.0])
    very_most_frequent = get_by_frequency(count_dict, num_samples, band=[0.5,1.0])

    lineage_dict = get_lineage_dict(in_metadata,downsample_lineage_size)

    outgroups = parse_outgroups(outgroup_file)
    indexed_fasta = SeqIO.index(in_fasta, "fasta")

    with open(in_metadata, 'r', newline = '') as csv_in, \
        open(out_fasta, 'w', newline = '') as fa_out, \
        open(out_metadata, 'w', newline = '') as csv_out:

        reader = csv.DictReader(csv_in, delimiter=",", quotechar='\"', dialect = "unix")
        writer = csv.DictWriter(csv_out, fieldnames = reader.fieldnames, delimiter=",", quotechar='\"', quoting=csv.QUOTE_MINIMAL, dialect = "unix")
        writer.writeheader()

        for row in reader:
            fasta_header = row["sequence_name"]
            if fasta_header not in indexed_fasta:
                continue
            if original_num_seqs % 1000 == 0:
                now = datetime.datetime.now()
                print("%s Downsampled from %i seqs to %i seqs" %(str(now), original_num_seqs, len(sample_dict)))
            original_num_seqs += 1

            if fasta_header in outgroups or not should_downsample_row(row,downsample_date_excluded, downsample_included,
                                                                      downsample_lineage_size,lineage_dict):
                if fasta_header in outgroups:
                    row["why_excluded"]=""
                writer.writerow(row)
                if row["why_excluded"] in [None, "None", ""] and fasta_header in indexed_fasta:
                    seq_rec = indexed_fasta[fasta_header]
                    fa_out.write(">" + seq_rec.id + "\n")
                    fa_out.write(str(seq_rec.seq) + "\n")
                else:
                    print(row["why_excluded"], fasta_header, (fasta_header in indexed_fasta))
                continue

            muts = row["nucleotide_variants"].split("|")
            if len(muts) < max_diff:
                #if not row["why_excluded"]:
                #    row["why_excluded"] = "downsampled with diff threshold %i" %max_diff
                writer.writerow(row)
                continue

            found_close_seq = False

            samples = set()
            low_frequency_muts = [mut for mut in muts if mut not in most_frequent]
            if len(low_frequency_muts) == 0:
                low_frequency_muts = [mut for mut in muts if mut not in very_most_frequent]
                if len(low_frequency_muts) == 0:
                    low_frequency_muts = muts
            if len(low_frequency_muts) > max_diff + 1:
                low_frequency_muts = low_frequency_muts[:max_diff+1]
            for mut in low_frequency_muts:
                if mut in var_dict:
                    samples.update(var_dict[mut])
            if downsample_lineage_size:
                samples = list( samples & set(lineage_dict[row["lineage"]]) )

            for sample in samples:
                if num_unique(muts, sample_dict[sample]) <= max_diff:
                    found_close_seq = True
                    #if not row["why_excluded"]:
                    #    row["why_excluded"] = "downsampled with diff threshold %i" %max_diff
                    writer.writerow(row)
                    break
            if not found_close_seq:
                sample_dict[fasta_header] = muts
                for mut in muts:
                    if mut not in var_dict:
                        var_dict[mut] = [fasta_header]
                    else:
                        var_dict[mut].append(fasta_header)
                row["why_excluded"] = ""
                writer.writerow(row)
                if fasta_header in indexed_fasta:
                    seq_rec = indexed_fasta[fasta_header]
                    fa_out.write(">" + seq_rec.id + "\n")
                    fa_out.write(str(seq_rec.seq) + "\n")

    now = datetime.datetime.now()
    print("%s Downsampled from %i seqs to %i seqs" %(str(now), original_num_seqs, len(sample_dict)))
    # return sample_dict.keys()

# def main():
#     args = parse_args()
#     subsample = downsample(args.in_metadata, args.out_metadata, args.in_fasta, args.out_fasta, args.diff, args.outgroups, args.downsample_date_excluded, args.downsample_included, args.downsample_lineage_size)

# if __name__ == '__main__':
#     main()


import sys

outputfile = sys.argv[1]

nameToLineage = dict()

class Lineage:
	def __init__(self, name):
		self.name = name
		self.precisions = []
		self.recalls = []
		self.f1s = []
		self.supports = []

	def printStats(self):
		avgPrec = sum(self.precisions) / len(self.precisions)
		avgRec = sum(self.recalls) / len(self.recalls)
		avgF1 = sum(self.f1s) / len(self.f1s) 
		totalSupport = sum(self.supports)

		print(self.name + "," + str(avgPrec) + "," + str(avgRec) + "," + str(avgF1) + "," + str(totalSupport))

with open(outputfile, 'r') as f:
	for line in f:
		line = line.strip()
		split = line.split()

		if "macro avg" in line or "weighted avg" in line:

			name = split[0] + " " + split[1]

			if name not in nameToLineage:
				nameToLineage[name] = Lineage(name)

			nameToLineage[name].precisions.append(float(split[2]))
			nameToLineage[name].recalls.append(float(split[3]))
			nameToLineage[name].f1s.append(float(split[4]))
			nameToLineage[name].supports.append(int(split[5]))

		if len(split) == 5 and ":" not in line and "read" not in line:
			name = split[0]

			if name not in nameToLineage:
				nameToLineage[name] = Lineage(name)

			nameToLineage[name].precisions.append(float(split[1]))
			nameToLineage[name].recalls.append(float(split[2]))
			nameToLineage[name].f1s.append(float(split[3]))
			nameToLineage[name].supports.append(int(split[4]))

print("lineage,precision,recall,f1_score,support")

for key in nameToLineage:
	if "macro" not in key and "weighted" not in key:
		nameToLineage[key].printStats()

nameToLineage["macro avg"].printStats()
nameToLineage["weighted avg"].printStats()



import csv
import pickle
import collections
from Bio import SeqIO
from Bio.Align import MultipleSeqAlignment
from Bio.Align.AlignInfo import SummaryInfo

## outputs a dataframe of positions to input in training

def get_lineage_cns50_sites(lineage, lineage_taxa, sequences_index, reference_sequence): 
            
    ### output of list of sequence_IDs for the given lineage
    
    lineage_seqs = MultipleSeqAlignment([])
    for taxon in lineage_taxa:
        lineage_seqs.append(sequences_index[taxon])

    info = SummaryInfo(lineage_seqs)
    consensus_sequence =  info.gap_consensus(
    threshold=0.50, 
    ambiguous='N')
    
    nuc_position = []
    for i in range(len(consensus_sequence)):
        if reference_sequence[i] != "N":
            if consensus_sequence[i] != "N":
                if consensus_sequence[i] != reference_sequence[i]:
                    nuc_position.append(i)
    return nuc_position

def get_relevant_positions(designation_file,seq_file,ref_file,outfile):
    reference = SeqIO.read(ref_file, "fasta")
    sequences_index = SeqIO.index(seq_file, "fasta")
    lineage_designations = collections.defaultdict(list)
    lineage_set = set()

    with open(designation_file, "r") as f:
        reader = csv.DictReader(f)
        for row in reader:
            lineage_designations[row["lineage"]].append(row["sequence_name"])
            lineage_set.add(row["lineage"])
    
    final_positions = set()
    for lineage in lineage_set:
        print(f"Getting positions for lineage {lineage}")
        positions = get_lineage_cns50_sites(lineage, lineage_designations[lineage], sequences_index, reference)
        print(f"\tFound {len(positions)}")
        for i in positions:
            final_positions.add(i)

    with open(outfile, 'wb') as pickle_file:
        pickle.dump(final_positions, pickle_file)


from Bio import SeqIO
import os
import csv
import sys

import collections
import hashlib
import collections
import csv

def version_from_init(init_file):
    version=None
    with open(init_file, "r") as fr:
        for l in fr:
            if l.startswith("__version__"):
                l = l.rstrip("\n")
                version = l.split('=')[1]
                version = version.replace('"',"").replace(" ","")
                break
    return version

def get_pango_version(pango_path):
    version =""

    for r,d,f in os.walk(pango_path):
        for fn in f:
            if fn == "__init__.py":
                version = version_from_init(os.path.join(r, fn))
                if not version:
                    continue
    print("Pango version is:", version)

    if not version:
        sys.sterr.write("No version found at pango path")
        sys.exit(-1)
    else:
        return version

def get_hash_string(record):
    seq = str(record.seq).upper().encode()
    hash_object = hashlib.md5(seq)
    hash_string = hash_object.hexdigest()
    return hash_string

def get_dict(in_csv,name_column,data_column):
    this_dict = {}
    with open(in_csv,"r") as f:
        reader = csv.DictReader(f)
        for row in reader:
            this_dict[row[name_column]] = row[data_column]
    return this_dict

def add_to_hash(seq_file):
    hash_map = {}
    seq_hash = {}
    for record in SeqIO.parse(seq_file, "fasta"):
        seq = str(record.seq).upper().encode()
        hash_object = hashlib.md5(seq)
        hash_map[hash_object.hexdigest()] = record.id
        seq_hash[str(record.seq)] = record.id
    return hash_map,seq_hash


from setuptools import setup, find_packages
import glob
import os
import pkg_resources
# Note: the _program variable is set in __init__.py.
# it determines the name of the package/final command line tool.
from pangoLEARN import __version__, _program

setup(name='pangoLEARN',
      version=__version__,
      packages=find_packages(),
      scripts=[],
      package_data={'pangoLEARN':['data/*']},
      description='trained lineage data',
      url='https://github.com/cov-lineages/pangoLEARN',
      author='cov-lineages group',
      author_email='aine.otoole@ed.ac.uk',
      entry_points="""
      [console_scripts]
      {program} = pangoLEARN.command:main
      """.format(program = _program),
      include_package_data=True,
      keywords=[],
      zip_safe=False)